
#include <packet32.h>
#include "netAnalyzer.h"
#include <iostream>
#include <fstream>
#include <deque>
#include <windows.h>
#include <stdio.h>

//#define DEBUG_TEXT

#define NETANA_API_NAME TEXT("netANALYZER_API.dll")
#define TIMEOUT_MS               100

void APIENTRY NetAnalyzerStatusCallback(uint32_t ulCaptureState, uint32_t ulCaptureError, void* pvUser);
void APIENTRY NetAnalyzerDataCallback(void* pvBuffer, uint32_t ulDataSize, void* vdata);
void NetAnalyzerInit(netAna_ADAPTER* netAnaAdapter);

using namespace std;

/* stores the value of the last device scan, to ensure a scan is not done too often in a short time, which would delay Wireshark startup */
uint64_t g_ullLastScanTime = 0;
#define DEV_SCAN_TIMOUT      10

#define FIFO_ENTRY_SIZE      sizeof(NETANA_FRAME_HEADER_T)

BOOLEAN NetAnaLoadApiDll(NETANADRIVER_FUNCTIONS *pfnFnc)
{
  BOOLEAN fRet = FALSE;

  // try to load the NetAnalyzer-API
  if (NULL == (pfnFnc->hDll = LoadLibrary(NETANA_API_NAME)) )
  {
      // Unable to load connector DLL
      fRet = FALSE;
  }
  else
  {
   // Read the netX API driver functions from the driver DLL
    if ( ((pfnFnc->pfn_netana_open_device           = ((PFN_netana_open_device)           GetProcAddress( pfnFnc->hDll, "netana_open_device")))                 == NULL) ||
         ((pfnFnc->pfn_netana_close_device          = ((PFN_netana_close_device)          GetProcAddress( pfnFnc->hDll, "netana_close_device")))                == NULL) ||
         ((pfnFnc->pfn_netana_driver_information    = ((PFN_netana_driver_information)    GetProcAddress( pfnFnc->hDll, "netana_driver_information")))          == NULL) ||
         ((pfnFnc->pfn_netana_device_info           = ((PFN_netana_device_info)           GetProcAddress( pfnFnc->hDll, "netana_device_info")))                 == NULL) ||
         //((pfnFnc->pfn_netana_get_error_description = ((PFN_netana_get_error_description) GetProcAddress( pfnFnc->hDll, "netana_get_error_description")))       == NULL) ||
         ((pfnFnc->pfn_netana_enum_device           = ((PFN_netana_enum_device)           GetProcAddress( pfnFnc->hDll, "netana_enum_device")))                 == NULL) ||
         ((pfnFnc->pfn_netana_mngmt_exec_cmd        = ((PFN_netana_mngmt_exec_cmd)        GetProcAddress( pfnFnc->hDll, "netana_mngmt_exec_cmd")))              == NULL) ||
         //((pfnFnc->pfn_netana_file_info           = ((PFN_NETANA_FILE_INFO)             GetProcAddress( pfnFnc->hDll, "netana_file_info")))                   == NULL) ||
         //((pfnFnc->pfn_netana_get_filter            = ((PFN_netana_get_filter)            GetProcAddress( pfnFnc->hDll, "netana_get_filter")))                  == NULL) ||
         //((pfnFnc->pfn_netana_set_filter            = ((PFN_netana_set_filter)            GetProcAddress( pfnFnc->hDll, "netana_set_filter")))                  == NULL) ||
         //((pfnFnc->pfn_netana_get_gpio_mode         = ((PFN_netana_get_gpio_mode)         GetProcAddress( pfnFnc->hDll, "netana_get_gpio_mode")))               == NULL) ||
         ((pfnFnc->pfn_netana_set_gpio_mode         = ((PFN_netana_set_gpio_mode)         GetProcAddress( pfnFnc->hDll, "netana_set_gpio_mode")))               == NULL) ||
         ((pfnFnc->pfn_netana_set_gpio_voltage      = ((PFN_netana_set_gpio_voltage)      GetProcAddress( pfnFnc->hDll, "netana_set_gpio_voltage")))            == NULL) ||
         //((pfnFnc->pfn_netana_get_portstat          = ((PFN_netana_get_portstat)          GetProcAddress( pfnFnc->hDll, "netana_get_portstat")))                == NULL) ||
         //((pfnFnc->pfn_netana_get_state             = ((PFN_netana_get_state)             GetProcAddress( pfnFnc->hDll, "netana_get_state")))                   == NULL) ||
         ((pfnFnc->pfn_netana_access_phy_reg        = ((PFN_netana_access_phy_reg)        GetProcAddress( pfnFnc->hDll, "netana_access_phy_reg")))              == NULL) ||
         //((pfnFnc->pfn_netana_set_filelist        = ((PFN_NETANA_SET_FILE_LIST)         GetProcAddress( pfnFnc->hDll, "netana_set_filelist")))                == NULL) ||
         ((pfnFnc->pfn_netana_start_capture         = ((PFN_netana_start_capture)         GetProcAddress( pfnFnc->hDll, "netana_start_capture")))               == NULL) ||
         ((pfnFnc->pfn_netana_stop_capture          = ((PFN_netana_stop_capture)          GetProcAddress( pfnFnc->hDll, "netana_stop_capture")))                == NULL)
         //((pfnFnc->pfn_netana_resync_time           = ((PFN_netana_resync_time)           GetProcAddress( pfnFnc->hDll, "netana_resync_time")))                 == NULL) ||
         //((pfnFnc->pfn_netana_set_gpio_output       = ((PFN_netana_set_gpio_output)       GetProcAddress( pfnFnc->hDll, "netana_set_gpio_output")))             == NULL) ||
         //((pfnFnc->pfn_netana_config_pi_controller  = ((PFN_netana_config_pi_controller)  GetProcAddress( pfnFnc->hDll, "netana_config_pi_controller")))        == NULL)
         )
   {
     // Error reading the connector functions from the DLL
     fRet = FALSE;
   }
   else
   {
    // Reading was successful
    fRet = TRUE;
   }
  }
  return fRet;
}

int NetAnalyzerCountDevices(NETANADRIVER_FUNCTIONS NetAnaPfnFnc)
{
  NETANA_DRIVER_INFORMATION_T ptDrvInfo = {0};
  NETANA_MNGMT_DEV_SCAN_IN_T  tInputParam;
  NETANA_MNGMT_DEV_SCAN_OUT_T tReturnVal;
  uint64_t ullNow;

  tInputParam.pfnCallBack = NULL;
  tInputParam.pvUser      = NULL;

  // scan for devices, but only in defined intervals, otherwise skip scan
  ullNow = _time64(NULL);
#ifdef DEBUG_TEXT
  cout << "NetAnalyzerCountDevices: ullNow: " << ullNow << " g_ullLastScanTime: " << g_ullLastScanTime << endl;
#endif
  if((g_ullLastScanTime+DEV_SCAN_TIMOUT) <= ullNow)
  {
#ifdef DEBUG_TEXT
    cout << "doing device scan" << endl;
#endif
    g_ullLastScanTime = ullNow;
    if(NETANA_NO_ERROR != NetAnaPfnFnc.pfn_netana_mngmt_exec_cmd( NETANA_MNGMT_CMD_DEV_SCAN, (void*)&tInputParam, sizeof(tInputParam), &tReturnVal, sizeof(tReturnVal)))
    {
      return 0;
    }
#ifdef DEBUG_TEXT
    cout << "end device scan, time: " << _time64(NULL) << endl;
#endif
  }
  else
  {
#ifdef DEBUG_TEXT
    cout << "skipped device scan, time: " << _time64(NULL) << endl;
#endif
  }
  // scan for devices
  if(NETANA_NO_ERROR != NetAnaPfnFnc.pfn_netana_driver_information(sizeof(ptDrvInfo), &ptDrvInfo))
    {
          return 0;
    }

#ifdef DEBUG_TEXT
    cout << "ulCardCnt: " << ptDrvInfo.ulCardCnt << ", time: " << _time64(NULL) << endl;
#endif

  return ptDrvInfo.ulCardCnt;
}

CHAR* NetAnalyzerGetName(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, int number)
{
  NETANA_DEVICE_INFORMATION_T tDevInfo = {0};

#ifdef DEBUG_TEXT
    cout << "NetAnalyzerGetName start, time: " << _time64(NULL) << endl;
#endif

  // scan for devices
  if(NETANA_NO_ERROR != NetAnaPfnFnc.pfn_netana_enum_device(number, sizeof(tDevInfo), &tDevInfo))
  {
#ifdef DEBUG_TEXT
    cout << "NetAnalyzerGetName end with error, time: " << _time64(NULL) << endl;
#endif
        return 0;
  }

#ifdef DEBUG_TEXT
    cout << "NetAnalyzerGetName end, time: " << _time64(NULL) << endl;
#endif
  return _strdup((char*)tDevInfo.szDeviceName);
}


/*
** Reads additional capture settings from registry, if existend and applies them to the current capture
*/
void NetAnalyzerGetSettingsFromRegistry(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, netAna_ADAPTER* netAnaAdapter)
{
  HKEY    hkCaptSettings;
  LONG    ulErrCodeReg = 0xffffffff;
  LONG    ulErrCodeNetAna = 0xffffffff;

  ulErrCodeReg = RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\Hilscher\\netANALYZER_WinPCAP\\"), 0, KEY_SET_VALUE | KEY_READ, &hkCaptSettings);
  if(ulErrCodeReg == ERROR_SUCCESS)
  {
    #define GPIO_CNT  4
    uint32_t ulMode[GPIO_CNT] = {0};
    uint32_t ulCaptureTriggers[GPIO_CNT] = {0};
    uint32_t ulEndDelay[GPIO_CNT] = {0};
    uint32_t ulGpioSelMsk = 0;
    uint32_t ulGpioVoltageSel = 0;

    uint32_t  ulData;
    uint32_t  ulDataSize = sizeof(ulData);
    wchar_t lpValueName[100];

#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: found registry key " << ", time:  " << _time64(NULL) << endl;
#endif

    // read all values for GPIOs from registry
    for(uint32_t ulIdx = 0; ulIdx < GPIO_CNT; ulIdx++)
    {

      swprintf_s(&lpValueName[0], sizeof(lpValueName), TEXT("ulMode%d"), ulIdx);
      ulErrCodeReg = RegQueryValueEx(hkCaptSettings, lpValueName, NULL, NULL, (LPBYTE)&ulData, (LPDWORD)&ulDataSize);
      if(ulErrCodeReg == ERROR_SUCCESS) ulMode[ulIdx] = ulData; else break;

      swprintf_s(&lpValueName[0], sizeof(lpValueName), TEXT("ulCaptureTriggers%d"), ulIdx);
      ulErrCodeReg = RegQueryValueEx(hkCaptSettings, lpValueName, NULL, NULL, (LPBYTE)&ulData, (LPDWORD)&ulDataSize);
      if(ulErrCodeReg == ERROR_SUCCESS) ulCaptureTriggers[ulIdx] = ulData; else break;

      swprintf_s(&lpValueName[0], sizeof(lpValueName), TEXT("ulEndDelay%d"), ulIdx);
      ulErrCodeReg = RegQueryValueEx(hkCaptSettings, lpValueName, NULL, NULL, (LPBYTE)&ulData, (LPDWORD)&ulDataSize);
      if(ulErrCodeReg == ERROR_SUCCESS) ulEndDelay[ulIdx] = ulData; else break;
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: found gpio settings " << ", time:  " << _time64(NULL) << endl;
#endif
    }

    if(ulErrCodeReg == ERROR_SUCCESS)
    {
      ulErrCodeReg = RegQueryValueEx(hkCaptSettings, TEXT("ulGpioSelMsk"), NULL, NULL, (LPBYTE)&ulData, (LPDWORD)&ulDataSize);
      if(ulErrCodeReg == ERROR_SUCCESS)
      {
        ulGpioSelMsk = ulData;

        ulErrCodeReg = RegQueryValueEx(hkCaptSettings, TEXT("ulGpioVoltageSel"), NULL, NULL, (LPBYTE)&ulData, (LPDWORD)&ulDataSize);
        if(ulErrCodeReg == ERROR_SUCCESS) ulGpioVoltageSel = ulData;

      }
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: found voltage settings " << ", time:  " << _time64(NULL) << endl;
#endif
    }

    if(ulErrCodeReg == ERROR_SUCCESS)
    {
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: config gpio " << ", time:  " << _time64(NULL) << endl;
#endif
      // all values read successfully, apply to netANALYZER
      for(uint32_t ulIdx = 0; ulIdx < GPIO_CNT; ulIdx++)
      {
        NETANA_GPIO_MODE_T  tGpioMode;

        tGpioMode.ulMode = ulMode[ulIdx];
        tGpioMode.uData.tTrigger.ulCaptureTriggers = ulCaptureTriggers[ulIdx];
        tGpioMode.uData.tTrigger.ulEndDelay = ulEndDelay[ulIdx];

        ulErrCodeNetAna = NetAnaPfnFnc.pfn_netana_set_gpio_mode(netAnaAdapter->netAnaHandle, ulIdx, &tGpioMode);
        if(ulErrCodeNetAna != 0) break;
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: gpio done " << ", time:  " << _time64(NULL) << endl;
#endif
      }

      if(ulErrCodeNetAna == 0)
      {
        ulErrCodeNetAna = NetAnaPfnFnc.pfn_netana_set_gpio_voltage(netAnaAdapter->netAnaHandle, ulGpioSelMsk, ulGpioVoltageSel);
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: gpio voltage done " << ", time:  " << _time64(NULL) << endl;
#endif
      }

      if(ulErrCodeNetAna != 0)
      {
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: error during setting capture settings, error: " << ulErrCodeNetAna << ", time:  " << _time64(NULL) << endl;
#endif
      }

    }
    else
    {
#ifdef DEBUG_TEXT
      cout << "NetAnalyzerGetSettingsFromRegistry: error during reading of registry, error: " << ulErrCodeReg << ", time:  " << _time64(NULL) << endl;
#endif
    }

    // if everything went fine until here, configure PHYs, otherwise skip
    if((ulErrCodeReg == ERROR_SUCCESS) && (ulErrCodeNetAna == 0))
    {
      #define PORT_CNT  4
      uint32_t ulPhyMode[PORT_CNT] = {0};

      // read all values for PHYs from registry
      for(uint32_t ulIdx = 0; ulIdx < PORT_CNT; ulIdx++)
      {
        swprintf_s(&lpValueName[0], sizeof(lpValueName), TEXT("ulPhyMode%d"), ulIdx);
        ulErrCodeReg = RegQueryValueEx(hkCaptSettings, lpValueName, NULL, NULL, (LPBYTE)&ulData, (LPDWORD)&ulDataSize);
        if(ulErrCodeReg == ERROR_SUCCESS) ulPhyMode[ulIdx] = ulData;
        else ulPhyMode[ulIdx] = 0;  // default is auto-detect

#ifdef DEBUG_TEXT
          cout << "NetAnalyzerGetSettingsFromRegistry: found PHY settings " << ", time:  " << _time64(NULL) << endl;
#endif
      }


#ifdef DEBUG_TEXT
      cout << "NetAnalyzerGetSettingsFromRegistry: config PHYs " << ", time:  " << _time64(NULL) << endl;
#endif
      // all values read successfully, apply to netANALYZER
      for(uint8_t ulIdx = 0; ulIdx < PORT_CNT; ulIdx++)
      {
        #define SPEED_AUTO_VALUE 0x1000
        #define SPEED_10_VALUE   0x0000
        #define SPEED_100_VALUE  0x2000
        #define REG_BASIC_CRTL   0

        uint16_t usPhyBits;

        switch(ulPhyMode[ulIdx])
        {
          case 100:
            usPhyBits = SPEED_100_VALUE;
            break;
          case 10:
            usPhyBits = SPEED_10_VALUE;
            break;
          default:
            usPhyBits = SPEED_AUTO_VALUE;
            break;
        }

        ulErrCodeNetAna = NetAnaPfnFnc.pfn_netana_access_phy_reg(netAnaAdapter->netAnaHandle, NETANA_PHY_DIRECTION_WRITE, ulIdx, REG_BASIC_CRTL, &usPhyBits, 1000);
        if(ulErrCodeNetAna != 0) break;
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerGetSettingsFromRegistry: PHY done " << ", time:  " << _time64(NULL) << endl;
#endif

        if(ulErrCodeNetAna != 0)
        {
#ifdef DEBUG_TEXT
          cout << "NetAnalyzerGetSettingsFromRegistry: error during setting PHY settings, error: " << ulErrCodeNetAna << ", time:  " << _time64(NULL) << endl;
#endif
        }

      }

    }

    RegSetValueEx(hkCaptSettings, TEXT("ulErrCodeNetAna"), 0, REG_DWORD, (LPBYTE)&ulErrCodeNetAna, sizeof(ulErrCodeNetAna));
    RegSetValueEx(hkCaptSettings, TEXT("ulErrCodeReg"), 0, REG_DWORD, (LPBYTE)&ulErrCodeReg, sizeof(ulErrCodeNetAna));

    RegCloseKey(hkCaptSettings);

  }
  else
  {
#ifdef DEBUG_TEXT
    cout << "NetAnalyzerGetSettingsFromRegistry: key not found, error: " << ulErrCodeReg << ", time:  " << _time64(NULL) << endl;
#endif
  }

}


/*
** Returns NULL if theres a error
*/
BOOLEAN NetAnalyzerOpenAdapter(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, netAna_ADAPTER* netAnaAdapter, PCHAR AdapterName)
{
  uint64_t ullReferenceTime = 0;

#ifdef DEBUG_TEXT
        cout << "NetAnalyzerOpenAdapter";
#endif

    NetAnalyzerInit(netAnaAdapter);

  // open the device
  if(NETANA_NO_ERROR != NetAnaPfnFnc.pfn_netana_open_device(AdapterName, &netAnaAdapter->netAnaHandle))
  {
    return FALSE;
  }

  // get additonal capture settings from registry
  NetAnalyzerGetSettingsFromRegistry(NetAnaPfnFnc, netAnaAdapter);

  //start capture
  ullReferenceTime = _time64(NULL) * 1000 * 1000 * 1000;
  if(NETANA_NO_ERROR != NetAnaPfnFnc.pfn_netana_start_capture(netAnaAdapter->netAnaHandle,
                                                   0,
                                                   0xF,
                                                   NETANA_MACMODE_ETHERNET,
                                                   ullReferenceTime,
                                                   NetAnalyzerStatusCallback,
                                                   NetAnalyzerDataCallback,
                                                   netAnaAdapter))
  {
    return FALSE;
  }
  return TRUE;
}

void NetAnalyzerInit(netAna_ADAPTER* netAnaAdapter)
{
  netAnaAdapter->curQueSize = 0;
  netAnaAdapter->startCapture = FALSE;
  netAnaAdapter->received = 0;
  netAnaAdapter->dropped = 0;
  netAnaAdapter->fOverFlowActive = 0;
  netAnaAdapter->captured = 0;
  netAnaAdapter->hReadEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

#ifdef DEBUG_TEXT
    cout << "NetAnalyzerInit, time: " << _time64(NULL) << endl;
#endif

 for(int i=0;i<8;i++)
  {
       netAnaAdapter->avRxPorts[i] = new(std::deque<DEC_NETANA_ETH_FRAME_Ttag>);
  }
}

BOOLEAN NetAnalyzerCloseAdapter(NETANADRIVER_FUNCTIONS NetAnaPfnFnc, netAna_ADAPTER* netAnaAdapter)
{
#ifdef DEBUG_TEXT
        cout << "NetAnalyzerCloseAdapter";
#endif

    // stop capture
  NetAnaPfnFnc.pfn_netana_stop_capture(netAnaAdapter->netAnaHandle);
  // close device
  NetAnaPfnFnc.pfn_netana_close_device(netAnaAdapter->netAnaHandle);

  CloseHandle(netAnaAdapter->hReadEvent);
  for(int i=0;i<8;i++)
  {
    delete(netAnaAdapter->avRxPorts[i]);
    netAnaAdapter->avRxPorts[i] = 0;
  }
  return TRUE;
}

BOOLEAN NetAnalyzerSetMode(netAna_ADAPTER* netAnaAdapter, DWORD mode)
{
  if (mode != PACKET_MODE_CAPT)
    return FALSE;

  netAnaAdapter->mode = mode;
  return TRUE;
}

BOOLEAN NetAnalyzerSetReadTimeout(netAna_ADAPTER* netAnaAdapter, DWORD readTimeout)
{
    if (readTimeout == 0)
        readTimeout = INFINITE;
    else if (readTimeout == -1)
            readTimeout = 0;

  netAnaAdapter->readTimeout = readTimeout;
    return TRUE;
}

BOOLEAN NetAnalyzerSetMinToCopy(netAna_ADAPTER* netAnaAdapter, DWORD minToCopy)
{
    netAnaAdapter->minToCopy = minToCopy;
    return TRUE;
}

HANDLE NetAnalyzerGetReadEvent(netAna_ADAPTER* netAnaAdapter)
{
    return netAnaAdapter->hReadEvent;
}

BOOLEAN NetAnalyzerSetBufferSize(netAna_ADAPTER* netAnaAdapter, DWORD BufferSize)
{
  netAnaAdapter->BufferSize = BufferSize;
  return TRUE;
}

BOOLEAN NetAnalyzerReceivePacket(netAna_ADAPTER* netAnaAdapter, PUCHAR Buffer,UINT maxLength, DWORD *actualLength)
{
    int smallest;
  int padding;
  *actualLength = 0;
    int64_t currentWinTime = GetTickCount();
  uint8_t* pCurrentPos = Buffer;
    int found;
  netAnaAdapter->startCapture = TRUE;
  std::deque<DEC_NETANA_ETH_FRAME_Ttag>        *avRxPorts[8];

  if(netAnaAdapter->status == NETANA_CAPTURE_STATE_STOP_PENDING )
      return FALSE;

  for(int i=0;i<8;i++)
    avRxPorts[i] = (std::deque<DEC_NETANA_ETH_FRAME_Ttag>*)netAnaAdapter->avRxPorts[i];

  ResetEvent(netAnaAdapter->hReadEvent);

    do
    {
        /* lock queues against filling */
        EnterCriticalSection(&netAnaAdapter->tcsGetOldestLock);

        // find next frame which may be timed out
        found = 0;
        smallest = 0;
    padding = 0;
        for (int i=0; i <= 7; i++)
        {
            if(!avRxPorts[i]->empty())
            {
                if(!avRxPorts[smallest]->empty())
                {
                    if (avRxPorts[i]->at(0).tHeader.ullTimestamp < avRxPorts[smallest]->at(0).tHeader.ullTimestamp)
                        smallest = i;
                }
                else
                    smallest = i;
            }
        }

        if (!avRxPorts[smallest]->empty())
        {
            // check if distance to previous frame is at least timeout-periode or if no new frame was captured
            // within this time
            uint64_t ullFrameSaveTime = avRxPorts[smallest]->at(0).tHeader.ullTimestamp + (TIMEOUT_MS*1000000);
            int64_t  ullFlushTime = netAnaAdapter->llLatestWinTime + TIMEOUT_MS;

            if ((netAnaAdapter->ullNewestFrameTime > ullFrameSaveTime) ||
                  (currentWinTime > ullFlushTime))
            {
                // call back to user with oldest frame
                DEC_NETANA_ETH_FRAME_T* ptFrame = &avRxPorts[smallest]->at(0);
        uint32_t ulFrameLen = (ptFrame->tHeader.ulHeader & NETANA_FRAME_HEADER_LENGTH_MSK) >> NETANA_FRAME_HEADER_LENGTH_SRT;

        if((4 - (ulFrameLen % 4)) != 4)
          {
             padding = (4-(ulFrameLen % 4));
          }
        UINT newLength = ((UINT)*actualLength + ulFrameLen + sizeof(ptFrame->tHeader) + sizeof(struct bpf_hdr) + padding);

        switch(netAnaAdapter->mode)
        {
        case PACKET_MODE_CAPT:
          if( newLength < maxLength)
          {
            // copy BFP to buffer
            struct bpf_hdr *bhp = (struct bpf_hdr*)pCurrentPos;
            bhp->bh_hdrlen = sizeof(struct bpf_hdr);
            bhp->bh_caplen = ulFrameLen + sizeof(ptFrame->tHeader.ulHeader);
            bhp->bh_datalen = ulFrameLen + sizeof(ptFrame->tHeader.ulHeader);
            bhp->bh_tstamp.tv_sec = (long)((uint64_t)ptFrame->tHeader.ullTimestamp / 1000000000);
            bhp->bh_tstamp.tv_usec = (long)((uint64_t)ptFrame->tHeader.ullTimestamp - (uint64_t)bhp->bh_tstamp.tv_sec *1000000000) ;
            bhp->bh_tstamp.tv_usec = bhp->bh_tstamp.tv_usec / 1000;
            pCurrentPos = pCurrentPos + bhp->bh_hdrlen;

            // copy tHeader to buffer
            memcpy (pCurrentPos, &ptFrame->tHeader, sizeof(ptFrame->tHeader.ulHeader));
            pCurrentPos = pCurrentPos + sizeof(ptFrame->tHeader.ulHeader);

            // copy data to buffer
            memcpy (pCurrentPos, &ptFrame->abData, ulFrameLen);
            pCurrentPos = pCurrentPos + ulFrameLen + padding;

            // Length of the buffer
            *actualLength += (DWORD)bhp->bh_hdrlen + (DWORD)bhp->bh_caplen + (DWORD)padding;
            avRxPorts[smallest]->pop_front();
            netAnaAdapter->curQueSize -= ulFrameLen;
                  found = 1;
          }
          break;
        case PACKET_MODE_STAT:
          break;
        case PACKET_MODE_MON:
          break;
        case PACKET_MODE_DUMP:
          break;
        }
            }
        }

        /* unlock queues for filling */
      LeaveCriticalSection(&netAnaAdapter->tcsGetOldestLock);

    } while (found == 1);


    EnterCriticalSection(&netAnaAdapter->tcsGetOldestLock);
    if( netAnaAdapter->curQueSize >= netAnaAdapter->minToCopy )
    {
        SetEvent(netAnaAdapter->hReadEvent);
    }
    LeaveCriticalSection(&netAnaAdapter->tcsGetOldestLock);

  return TRUE;
}

void NetAnalyzerInsertFrame(netAna_ADAPTER *netAnaAdapter, uint32_t ulPort, DEC_NETANA_ETH_FRAME_T* ptFrame, uint32_t ulFrameLen)
{
  DEC_NETANA_ETH_FRAME_T                        tCpyFrame;
  std::deque<DEC_NETANA_ETH_FRAME_Ttag>        *avRxPorts[8];

  for(int i=0;i<8;i++)
    avRxPorts[i] = (std::deque<DEC_NETANA_ETH_FRAME_Ttag>*)netAnaAdapter->avRxPorts[i];

  if(netAnaAdapter->startCapture == TRUE)
  {
    // get windows timestamp when last frame was received
    netAnaAdapter->llLatestWinTime = GetTickCount();
    netAnaAdapter->ullNewestFrameTime = ptFrame->tHeader.ullTimestamp;

    // first copy only valid data to temp buffer, then insert it into the list
    memcpy(&tCpyFrame, ptFrame, ulFrameLen+sizeof(NETANA_FRAME_HEADER_T));

    // lock this function against simultaneous access to vector via timeout and DataCallBack
    EnterCriticalSection(&netAnaAdapter->tcsGetOldestLock);
    avRxPorts[ulPort]->push_back(tCpyFrame);
    LeaveCriticalSection(&netAnaAdapter->tcsGetOldestLock);

    netAnaAdapter->curQueSize += ulFrameLen;
  }
}


void APIENTRY NetAnalyzerDataCallback(void* pvBuffer, uint32_t ulDataSize, void* vdata)
{
  netAna_ADAPTER *netAnaAdapter = (netAna_ADAPTER*)vdata;
  uint8_t*  pbBuffer   = (uint8_t*)pvBuffer;
  uint32_t  ulOffset   = 0;
  uint32_t  ulPort     = 0;

  while(ulOffset < ulDataSize)
  {
    DEC_NETANA_ETH_FRAME_T* ptFrame  = (DEC_NETANA_ETH_FRAME_T*)(pbBuffer + ulOffset);
    uint32_t ulFrameLen = (ptFrame->tHeader.ulHeader & NETANA_FRAME_HEADER_LENGTH_MSK) >> NETANA_FRAME_HEADER_LENGTH_SRT;

    // sort in, ports0..3 and gpio0..3
    ulPort = (ptFrame->tHeader.ulHeader & NETANA_FRAME_HEADER_PORT_MSK ) >> NETANA_FRAME_HEADER_PORT_SRT;
    if ((ptFrame->tHeader.ulHeader & NETANA_FRAME_HEADER_GPIOEVT_MSK ) >> NETANA_FRAME_HEADER_GPIOEVT_SRT != 00)
      ulPort += 4;

    netAnaAdapter->received++;

    if(netAnaAdapter->BufferSize >= (ulFrameLen + sizeof(NETANA_FRAME_HEADER_T) + netAnaAdapter->curQueSize + (2*FIFO_ENTRY_SIZE)))
    {
      // if previous overflow condition ended, put overflow recovery entry
      if (netAnaAdapter->fOverFlowActive == 1)
      {
        NETANA_FRAME_HEADER_T tFifoOverflowFrame;
        tFifoOverflowFrame.ulHeader = (NETANA_FRAME_HEADER_TYPE_VAL_FIFO_STATE << NETANA_FRAME_HEADER_TYPE_SRT) |
          (ulPort << NETANA_FRAME_HEADER_PORT_SRT) |
          (2 << NETANA_FRAME_HEADER_VERSION_SRT) |
          (NETANA_FRAME_HEADER_OVERFLOW_STATE_VAL_END << NETANA_FRAME_HEADER_OVERFLOW_STATE_SRT) |
          (NETANA_FRAME_HEADER_OVERFLOW_SRC_VAL_CAPDRV << NETANA_FRAME_HEADER_OVERFLOW_SRC_SRT);
        tFifoOverflowFrame.ullTimestamp = ptFrame->tHeader.ullTimestamp;

        NetAnalyzerInsertFrame(netAnaAdapter, ulPort, (DEC_NETANA_ETH_FRAME_T*) &tFifoOverflowFrame, 0);

      }
      netAnaAdapter->fOverFlowActive = 0;

      NetAnalyzerInsertFrame(netAnaAdapter, ulPort, ptFrame, ulFrameLen);
      netAnaAdapter->captured++;

    }
    else
    {
      netAnaAdapter->dropped++;

      // if overflow condition occured first time, put overflow entry
      if (netAnaAdapter->fOverFlowActive == 0)
      {
        NETANA_FRAME_HEADER_T tFifoOverflowFrame;

        tFifoOverflowFrame.ulHeader = (NETANA_FRAME_HEADER_TYPE_VAL_FIFO_STATE << NETANA_FRAME_HEADER_TYPE_SRT) |
          (ulPort << NETANA_FRAME_HEADER_PORT_SRT) |
          (2 << NETANA_FRAME_HEADER_VERSION_SRT) |
          (NETANA_FRAME_HEADER_OVERFLOW_STATE_VAL_START << NETANA_FRAME_HEADER_OVERFLOW_STATE_SRT) |
          (NETANA_FRAME_HEADER_OVERFLOW_SRC_VAL_CAPDRV << NETANA_FRAME_HEADER_OVERFLOW_SRC_SRT);
        tFifoOverflowFrame.ullTimestamp = ptFrame->tHeader.ullTimestamp;

        NetAnalyzerInsertFrame(netAnaAdapter, ulPort, (DEC_NETANA_ETH_FRAME_T*) &tFifoOverflowFrame, 0);
      }

      netAnaAdapter->fOverFlowActive = 1;
    }
    // Adjust Offset to next DWORD aligned address
    ulOffset += sizeof(NETANA_FRAME_HEADER_T) + ulFrameLen;
    while(ulOffset % 4)
      ++ulOffset;
  }

    EnterCriticalSection(&netAnaAdapter->tcsGetOldestLock);
    if( netAnaAdapter->curQueSize >= netAnaAdapter->minToCopy )
    {
        SetEvent(netAnaAdapter->hReadEvent);
    }
    LeaveCriticalSection(&netAnaAdapter->tcsGetOldestLock);
}

void APIENTRY NetAnalyzerStatusCallback(uint32_t ulCaptureState, uint32_t ulCaptureError, void* vdata)
{
  netAna_ADAPTER *netAnaAdapter = (netAna_ADAPTER*)vdata;
  (void) ulCaptureError;

  switch(ulCaptureState)
  {
  case NETANA_CAPTURE_STATE_OFF:
    netAnaAdapter->status = NETANA_CAPTURE_STATE_OFF;
#ifdef DEBUG_TEXT
        cout << "NETANA_CAPTURE_STATE_OFF";
#endif
    break;

  case NETANA_CAPTURE_STATE_START_PENDING:
    netAnaAdapter->status = NETANA_CAPTURE_STATE_START_PENDING;
#ifdef DEBUG_TEXT
        cout << "NETANA_CAPTURE_STATE_START_PENDING";
#endif
    break;

  case NETANA_CAPTURE_STATE_RUNNING:
    netAnaAdapter->status = NETANA_CAPTURE_STATE_RUNNING;
#ifdef DEBUG_TEXT
        cout << "NETANA_CAPTURE_STATE_RUNNING";
#endif
    break;

  case NETANA_CAPTURE_STATE_STOP_PENDING:
    netAnaAdapter->status = NETANA_CAPTURE_STATE_STOP_PENDING;
#ifdef DEBUG_TEXT
        cout << "NETANA_CAPTURE_STATE_STOP_PENDING";
#endif
    OutputDebugStringA("NetAnalyzer: Capture Stop Pending (Status-Callback)\n");
    break;

  default:
    break;
  }
}
